﻿using System;
using System.Collections.Generic;
using System.Text;
using SuperMap.Data;
using SuperMap.Desktop;
using Application = SuperMap.Desktop.Application;

namespace SuperMap.Desktop.Samples
{
     //Define the geometry object model ArrowDrawing, inherited from GeometryDrawing
    public class ArrowDrawing:GeometryDrawing
    {
        //Calculate the coordinates of corners of this arrow
        private Point2D CalculatePoint(Point2D basePoint, Double rotation, Double distance)
        {
            Point2D resultPoint = new Point2D();
            try
            {
                resultPoint.X = basePoint.X + distance * Math.Cos(rotation);
                resultPoint.Y = basePoint.Y + distance * Math.Sin(rotation);
            }
            catch (Exception ex)
            {
            	Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return resultPoint;
        }
        
        //Calculate the distance between two points
        private Double Distance(Point2D point1, Point2D point2)
        {
            Double disance = Math.Sqrt((point1.X - point2.X) * (point1.X - point2.X) + (point1.Y - point2.Y) * (point1.Y - point2.Y));
            return disance;
        }

        //Calculate the coordinates of points of this arrow, and return the arrow regionobject.
        private GeoRegion GetArrow(Point2D startPoint, Point2D endPoint,Double width)
        {
            GeoRegion result = null;
            try
            {
                Double length = Distance(startPoint, endPoint);
                Double slope = (startPoint.Y - endPoint.Y) / (startPoint.X - endPoint.X);
                Double rotation = Math.Atan(slope);
                if (startPoint.Y - endPoint.Y >= 0 && startPoint.X - endPoint.X >= 0)
                {
                    rotation = rotation + Math.PI;
                }
                if (startPoint.Y - endPoint.Y < 0 && startPoint.X - endPoint.X >= 0)
                {
                    rotation = rotation + Math.PI;
                }
                Point2D point1 = CalculatePoint(this.Parameters[0].Point2D, rotation + Math.PI / 2, width / 2);
                Point2D point7 = CalculatePoint(this.Parameters[0].Point2D, rotation - Math.PI / 2, width / 2);
                Point2D point2 = CalculatePoint(point1, rotation, length * 0.8);
                Point2D point3 = CalculatePoint(point2, rotation + Math.PI / 2, width / 2);
                Point2D point4 = endPoint;
                Point2D point6 = CalculatePoint(point7, rotation, length * 0.8);
                Point2D point5 = CalculatePoint(point6, rotation - Math.PI / 2, width / 2);

                result = new GeoRegion(new Point2Ds(new Point2D[] { point1, point2, point3, point4, point5, point6, point7 }));
            }
            catch (Exception ex)
            {
            	Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return result;
        }

        //Whether the object drawing is completed
        public override Boolean IsCompleted
        {
            get { return this.Parameters.Count == 3; }
        }

        // Validate whether the parameter is legal.
        public override bool VerifyInput(GeometryDrawingParameter newParam)
        {
            Boolean result = true;
            try
            {
                // Step 2: Check the validity
                if (this.Parameters.Count == 1)
                {
                    if (newParam.Point2D == Point2D.Empty ||
                    (this.Parameters.Count > 0 && newParam.Point2D == this.Parameters.LastItem.Point2D))
                    {
                        result = false;
                    }
                }
                //Step 3: it is illegal when the arrow width is 0
                else if (this.Parameters.Count == 2)
                {
                    if (newParam.Length == 0)
                    {
                        result = false;
                    }
                } 
            }
            catch (Exception ex)
            {
            	Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }      
            return result;
        }

        // Get the next parameter
        public override ParameterInputMethod GetNextParameterInputMethod()
        {
            ParameterInputMethod method = new ParameterInputMethod(DrawingParameterType.Coordinate);
            try
            {
                if (this.Parameters.Count == 2)
                {
                    method = new ParameterInputMethod(DrawingParameterType.Length);
                    Point2D point1 = this.Parameters[0].Point2D;
                    Point2D point2 = this.Parameters[1].Point2D;
                    Double angle = Math.Atan2(point1.Y - point2.Y,point1.X - point2.X);
                    method.BaseAngle = angle /  Math.PI*180 + 90;  //convert radians to angles
                    method.BasePoint2D = new Point2D((point1.X+point2.X)/2,(point1.Y+point2.Y)/2);
                }
            }
            catch (Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return method;
        }

        // Get the result
        public override Geometry GetGeometry()
        {
            Geometry result = null;
            try
            {
                if (this.Parameters.Count == 3)
                {
                    result = GetArrow(this.Parameters[0].Point2D, this.Parameters[1].Point2D, this.Parameters[2].Length);
                }
            }
            catch (Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return result;
        }

        // Get the preview
        public override GeometryDrawingTrackingResult GetTrackingGeometry(GeometryDrawingParameter previewParameter)
        {
            GeometryDrawingTrackingResult result = new GeometryDrawingTrackingResult();
            try
            {
                //Step 2
                if(this.Parameters.Count==1&&this.Parameters[0].Point2D != previewParameter.Point2D && previewParameter.Point2D!=Point2D.Empty)
                {
                    Point2D point1 = this.Parameters[0].Point2D;
                    Point2D point2 = previewParameter.Point2D;
                    Double length = Distance(point1, point2);
                    result.ResultGeoemtry = GetArrow(point1, point2, length / 5);
                }
                //Step 3
                else if (this.Parameters.Count == 2)
                {
                    if (previewParameter.Length!=0)
                    {
                        result.ResultGeoemtry = GetArrow(this.Parameters[0].Point2D, this.Parameters[1].Point2D, previewParameter.Length);
                    }
                }
            }
            catch (Exception ex)
            {
                Application.ActiveApplication.Output.Output(ex.StackTrace, InfoType.Exception);
            }
            return result;
        }
    }
}


